{
*****************************************************************************
*  Name    : S1D15G00.BAS                                                   *
*  Author  : David John Barker                                              *
*  Notice  : Copyright (c) 2007 Mecanique                                   *
*          : All Rights Reserved                                            *
*  Date    : 25/07/2006                                                     *
*  Version : 1.3                                                            *
*  Notes   : 1.3 add utils.bas for SetAllDigital                            *   
*          : 1.2 add #option GLCD_SETALLDIGITAL                             *   
*          : 1.1 Change WriteChar() to handle new 'non-compressed' x fonts  * 
*          : 1.0 Release                                                    *
*****************************************************************************
}
module S1D15G00

#define GLCD_PIXEL_08
#define GLCD_COLOR_08
#define GLCD_XY_08
include "Graphics.bas" 
include "system.bas" 
include "utils.bas"
 
// default module options - user options can override these values...
#option GLCD_RESET = PORTC.7     // reset pin
#option GLCD_DATA = PORTC.5      // data pin
#option GLCD_CLK = PORTC.3       // clock pin
#option GLCD_CS = PORTC.1        // CS pin
#option GLCD_ASPECT_RATIO = 100  // aspect ratio
#option GLCD_INIT_DELAY = 100    // initialisation delay (ms)
#option GLCD_SPI = SW            // use software SPI 

// by default, the module calls SetAllDigital in Initialize()
// set option false to exclude the call
#if IsOption(GLCD_SETALLDIGITAL) and not (GLCD_SETALLDIGITAL in (true, false))
   #error GLCD_SETALLDIGITAL, "Invalid option"
#endif
#option GLCD_SETALLDIGITAL = true

// validate Reset pin...
#if IsOption(GLCD_RESET) and not IsValidPortPin(GLCD_RESET) 
   #error GLCD_RESET, "Invalid option. RESET must be a valid port pin."
#endif

// validate Data pin...
#if IsOption(GLCD_DATA) and not IsValidPortPin(GLCD_DATA) 
   #error GLCD_DATA, "Invalid option. DATA must be a valid port pin."
#endif

// validate Clk pin...
#if IsOption(GLCD_CLK) and not IsValidPortPin(GLCD_CLK) 
   #error GLCD_CLK, "Invalid option. CLK must be a valid port pin."
#endif

// validate CS pin...
#if IsOption(GLCD_CS) and not IsValidPortPin(GLCD_CS) 
   #error GLCD_CS, "Invalid option. CS must be a valid port pin."
#endif

// validate initialisation delay...
#if IsOption(GLCD_INIT_DELAY)
   #if not (GLCD_INIT_DELAY in (0 to 1000))
      #error GLCD_INIT_DELAY, "Invalid option. GLCD initialize delay must be between 0 and 1000 (ms)."
   #endif
#endif 

// validate SPI option...
#if IsOption(GLCD_SPI) and not (GLCD_SPI in (SW, MSSP))
   #error GLCD_SPI, "Invalid option. SPI must be SW or MSSP (hardware)."
#endif 

// import SPI, if required...
#if GLCD_SPI = MSSP
include "SPI.bas"
#endif   

// S1D15G00 commands...   
const
   cmdDisCtl = $CA,
   cmdComScn = $BB,
   cmdOscOn = $D1,
   cmdSlpIn = $95,
   cmdSlpOut = $94,
   cmdVolCtr = $81,
   cmdPwtCtr = $20,
   cmdDatCtl = $BC,
   cmdRGBSet8 = $CE,
   cmdNOP = $25,
   cmdDisOn = $AF,
   cmdDisOff = $AE,
   cmdDisNOR = $A6,
   cmdDisInv = $A7,
   cmdVolUp = $D6,
   cmdVolDown = $D7,
   cmdPaSet = $75,
   cmdCaSet = $15,
   cmdRamWr = $5C,
   GLCDDelay = GLCD_INIT_DELAY     

// port and pin settings, these are brought into
// the program by using the above options...
dim        
   Data as GLCD_DATA.GLCD_DATA@,        // data pin 
   Clk as GLCD_CLK.GLCD_CLK@,           // clk pin
   CS as GLCD_CS.GLCD_CS@,              // CS pin
   Reset as GLCD_RESET.GLCD_RESET@,     // reset pin
   FContrast as byte                    // current contrast
   
// GLCD width and height...
public const
   GLCDWidth = 130,
   GLCDHeight = 130

// x, y position...
public dim
   Pos as TPosition,
   Contrast as FContrast   
 
{
****************************************************************************
* Name    : SPICommand (PRIVATE)                                           *
* Purpose : Write a command byte to GLCD                                   *
****************************************************************************
}   
#if GLCD_SPI = MSSP
sub SPICommand(pData as byte)
   CS = 0
   Clk = 0                           
   Data = 0
   Nop
   Clk = 1
   Nop
   Clk = 0
   
   SSPEN = 1               // turn hardware SPI back on
   SPI.WriteByte(pData)    // write 8 bits...
   ClrWDT
   SSPEN = 0               // shut off hardware SPI allowing direct access to SPI in/out pins  
   CS = 1                  // clock idles low
end sub


#else
sub SPICommand(pData as byte)
   dim Shift as byte

   // enable chip select, then clock
   // command type...
   CS = 0
   Clk = 0                           
   Data = 0 
   Clk = 1
   Nop
   
   // write 8 bits...
   Shift = 8
   repeat
      Clk = 0
      Data = pData.7
      pData = pData << 1
      dec(Shift)
      Clk = 1
   until Shift = 0
   
   // clock idles low, disable chip select...
   Clk = 0
   CS = 1 
   ClrWDT
end sub
#endif 
(*
sub SPICommand(pData as byte)
   dim Shift as byte

   // enable chip select, then clock
   // command type...
   CS = 0
   Clk = 0                           
   Data = 0
   Clk = 1
   Nop
   
   // write 8 bits...
   Shift = 8
   repeat
      Clk = 0
      Data = pData.7
      pData = pData << 1
      dec(Shift)
      Clk = 1
   until Shift = 0
   
   // clock idles low, disable chip select...
   Clk = 0
   CS = 1 
   ClrWDT
end sub
*)
{
****************************************************************************
* Name    : SPIData (PRIVATE)                                              *
* Purpose : Write a data byte to GLCD                                      *
****************************************************************************
}    
#if GLCD_SPI = MSSP
sub SPIData(pData as byte)

   CS = 0
   Clk = 0                           
   Data = 1
   Nop
   Clk = 1
   Nop
   Clk = 0
   
   SSPEN = 1               // turn hardware SPI back on
   SPI.WriteByte(pData)    // write 8 bits...
   ClrWDT
   SSPEN = 0               // shut off hardware SPI allowing direct access to SPI in/out pins
   CS = 1                  // clock idles low
end sub
#else
sub SPIData(pData as byte)
   dim Shift as byte

   // enable chip select, then clock
   // data type...
   CS = 0  
   Clk = 0  
   Data = 1
   Clk = 1 
   Nop
   
   // write 8 bits...
   Shift = 8
   repeat
      Clk = 0
      Data = pData.7
      pData = pData << 1
      dec(Shift)
      Clk = 1
   until Shift = 0
   
   // clock idles low, disable chip select...
   Clk = 0
   CS = 1
   ClrWDT
end sub
#endif 
(* 
sub SPIData(pData as byte)
   dim Shift as byte

   // enable chip select, then clock
   // data type...
   CS = 0  
   Clk = 0  
   Data = 1
   Clk = 1 
   Nop
   
   // write 8 bits...
   Shift = 8
   repeat
      Clk = 0
      Data = pData.7
      pData = pData << 1
      dec(Shift)
      Clk = 1
   until Shift = 0
   
   // clock idles low, disable chip select...
   Clk = 0
   CS = 1
   ClrWDT
end sub
*)
{
****************************************************************************
* Name    : WriteData (PRIVATE)                                            *
* Purpose : Write a series of data bytes to GLCD                           *
****************************************************************************
}     
compound sub WriteData(SPIData)
{
****************************************************************************
* Name    : MoveTo (PRIVATE)                                               *
* Purpose : Move to position x,y on the GLCD                               *
****************************************************************************
}     
sub MoveTo(pX, pY as byte)
   SPICommand(cmdCaSet)
   SPIData(pX)
   SPIData(GLCDWidth - 1)
   SPICommand(cmdPaSet)
   SPIData(pY + 2)
   SPIData(GLCDHeight + 1)
   SPICommand(cmdRamWr)   
end sub
{
****************************************************************************
* Name    : SetPixel                                                       *
* Purpose : Set pixel at pixel location x,y                                *
****************************************************************************
} 
public sub SetPixel(pX, pY as byte)
   MoveTo(pX, pY)
   SPIData(Pen.Color)
end sub
{
****************************************************************************
* Name    : SetContrast                                                    *
* Purpose : Set the GLCD contrast                                          *
****************************************************************************
} 
public sub SetContrast(pContrast as byte)
   dim Contrast as byte
   
   if pContrast < FContrast then
      Contrast = FContrast - pContrast
      while Contrast > 0
          SPICommand(cmdVolDown) 
          dec(Contrast)
      wend
   elseif pContrast > FContrast then
      Contrast = pContrast - FContrast
      while Contrast > 0
          SPICommand(cmdVolUp) 
          dec(Contrast)
      wend
   endif
   FContrast = pContrast
end sub
{
****************************************************************************
* Name    : Cls                                                            *
* Purpose : Clear the GLCD screen                                          *
****************************************************************************
} 
public sub Cls(pColor as TColor = $FF)
   dim Index as word

   MoveTo(0,0)
   Index = GLCDWidth * GLCDHeight
   repeat
      SPIData(pColor)
      dec(Index)
   until Index = 0
end sub
{
****************************************************************************
* Name    : MakeColor                                                      *
* Purpose : The S1D15G00 8 bit color scheme maps 3 bit R, G values and     *
*         : a 2 bit B value onto a 12 bit RGB. This routine allows you     *
*         : to create a color by specifying RGB values in the range 0..255 *
****************************************************************************
} 
public function MakeColor(pR, pG, pB as byte) as TColor
   result = (pR / 32 << 5) or (pG / 32 << 2) or (pB / 64)  
end function
{
****************************************************************************
* Name    : Initialize                                                     *
* Purpose : Configure the GLCD before use                                  *
****************************************************************************
} 
sub Initialize()

   #if (GLCD_SETALLDIGITAL)
   SetAllDigital()
   #endif

   FContrast = 0
   delayms(GLCDDelay)
   
   #if GLCD_SPI = MSSP
   SPI.SetAsMaster(spiOscDiv16)//(spiOscDiv4)
   SPI.SetClock(spiIdleLow, spiRisingEdge)
   SSPEN = 0   
   #endif    
   
   // set port direction...
   low(Data)
   low(Clk)
   high(CS)
   
   // reset...
   low(Reset)  
   delayms(1)
   Reset = 1
   delayms(1)
   
   // display control - Driving pattern, Duty, FRM...
   SPICommand(cmdDisCtl)
   WriteData($03,$20,$0C)

   // scan 1 to 80...
   SPICommand(cmdComScn)
   SPIData($01)
   
   // oscillator and sleep out...
   SPICommand(cmdOscOn)   
   SPICommand(cmdSlpOut)
   
   // contrast...
   SPICommand(cmdVolCtr)
   WriteData($00,$01)      
   
   // power control...
   SPICommand(cmdPwtCtr)
   SPIData($0F)
   delayms(100)
   
   // display inverse...
   SPICommand(cmdDisInv)  
 
   // data control...
   SPICommand(cmdDatCtl)
   WriteData($00,$00,$01)
   
   // color palette...
   SPICommand(cmdRGBSet8)
   WriteData($00,$02,$04,$06,$08,$0A,$0C,$0F)
   WriteData($00,$02,$04,$06,$08,$0A,$0C,$0F)
   WriteData($00,$05,$0A,$0F)

   delayms(100)
   SPICommand(cmdNOP)
   SPICommand(cmdDisOn)

   // set default contrast, user may have to adjust...
   SetContrast(158)
end sub
{
****************************************************************************
* Name    : SetRegion (PRIVATE)                                            *
* Purpose : Sets the GLCD scan region                                      *
****************************************************************************
} 
sub SetRegion(pX, pY, pWidth, pHeight as byte)
   SPICommand(cmdCaSet)
   SPIData(pX)
   SPIData(pX + pWidth - 1)
   SPICommand(cmdPaSet)
   SPIData(pY + 2)
   SPIData(pY + pHeight + 1)
   SPICommand(cmdRamWr)  
end sub
{
****************************************************************************
* Name    : Fill                                                           *
* Purpose : Fast fill screen area with brush color                         *
*         : Assumes pX1 < pX2 and pY1 < pY2                                *
****************************************************************************
} 
public sub Fill(pX1, pY1, pX2, pY2 as TXY)
   dim Width, Height as TXY
   dim Index as word

   Width = pX2 - pX1 + 1
   Height = pY2 - pY1 + 1
   SetRegion(pX1, pY1, Width, Height)
   Index = Width * Height
   repeat
      SPIData(Brush.Color)
      dec(Index)
   until Index = 0
end sub
{
****************************************************************************
* Name    : ReadImageByte (PRIVATE)                                        *
* Purpose : Read an image byte                                             *
****************************************************************************
} 
function ReadImageByte() as byte
   asm-
      TBLRD *+
      movff TABLAT, result
   end asm
end function
{
****************************************************************************
* Name    : SetImage (OVERLOAD)                                            *
* Purpose : Overrides the GLCD library native SetImage call with this      *
*         : routine which is better suited for rendering to the S1D15G00   *
*         : pImageAdr is the address of the image to render                *
****************************************************************************
} 
#define GLCD_SETIMAGE // override GLCD SetImage
public sub SetImage(pX, pY as TXY, pImageAdr as word)
   dim x, xRange as TXY
   dim TransColor as TColor
   dim ImageByte1, ImageByte2, ImageByte3 as byte
   dim Index, Range as word
   dim RLENum as byte
   dim LastPenColor as byte
   dim Image as TImage
   dim Ignore as boolean
   
   // get bitmap ID, width and height...
   TABLEPTR = pImageAdr
   Image.Header = ReadImageByte
   Image.Width = ReadImageByte
   Image.Height = ReadImageByte
   inc(TABLEPTR)
   
   // last pen color to restore pen, index starts
   // at 4 to account for header size...
   LastPenColor = Pen.Color
   Index = 0

   // 8 bit color...
   if Image.Is8Bit then
      Range = Image.Height * Image.Width
   
      // is it a transparent image...
      if Image.IsTransparent then
         Ignore = false
         MoveTo(pX,pY)
         x = pX
         xRange = pX + Image.Width

         // read the transparent pixel...
         if Image.IsRLE then
            inc(TABLEPTR)
         endif
         TransColor = ReadImageByte
         TABLEPTR = pImageAdr + 4
         
         // repeat for all the pixels in the image...
         repeat
            RLENum = 1
            if Image.IsRLE then
               RLENum = ReadImageByte
            endif
            Pen.Color = ReadImageByte
            repeat   
               
               // if it's a transparent pixel, skip...
               if Pen.Color = TransColor then
                  Ignore = true
               else 
                  // we have hit a non transparent pixel, so reset x, y
                  if Ignore then
                     SPICommand(cmdCaSet)
                     SPIData(x)
                     SPIData(pX + Image.Width)
                     SPICommand(cmdRamWr)
                     Ignore = false
                  endif  
                  SPIData(Pen.Color) 
               endif               
               
               // inc x, if greater than image width then reset
               // x and move to next y
               inc(x)
               if x >= xRange then
                  x = pX
                  inc(pY)
                  MoveTo(pX,pY)
               endif
               
               inc(Index)
               dec(RLENum)
            until RLENum = 0   
         until Index >= Range

      // image is not transparent, fast bit blit...   
      else
         SetRegion(pX, pY, Image.Width, Image.Height)
         repeat
            RLENum = 1
            if Image.IsRLE then
               RLENum = ReadImageByte
            endif
            Pen.Color = ReadImageByte
            repeat
               SPIData(Pen.Color)
               inc(Index)
               dec(RLENum)
            until RLENum = 0
         until Index >= Range
      endif

   // 12 bit, fast bit blit - note that 12 bit images must fall on an even x...
   elseif Image.Is12Bit then
      SPICommand(cmdDatCtl)
      WriteData($00,$00,$02)
      pX = pX and $FE
      SetRegion(pX, pY, Image.Width, Image.Height)
      Range = Image.Height * (Image.Width + Image.Width / 2)
      repeat
         RLENum = 1
         if Image.IsRLE then
            RLENum = ReadImageByte
         endif
         ImageByte1 = ReadImageByte
         ImageByte2 = ReadImageByte
         ImageByte3 = ReadImageByte
         repeat
            SPIData(ImageByte1)
            SPIData(ImageByte2)
            SPIData(ImageByte3)
            inc(Index, 3)
            dec(RLENum)
         until RLENum = 0
      until Index >= Range
      SPICommand(cmdDatCtl)
      WriteData($00,$00,$01)
   endif 
   Pen.Color = LastPenColor 
end sub
{
****************************************************************************
* Name    : SetImage (OVERLOAD)                                            *
* Purpose : Overrides the GLCD library native SetImage call with this      *
*         : routine which is better suited for rendering to the S1D15G00   *
*         : pImage is a constant array holding the image data to render    *
****************************************************************************
} 
public sub SetImage(pX, pY as TXY, byrefconst pImage() as byte)
   SetImage(pX, pY, @pImage)
end sub 
{
****************************************************************************
* Name    : CopyImage (OVERLOAD)                                           *
* Purpose : Copies pImage to location pX, pY. The part of the image you    *
*         : to render is specified by pX1, pY1, pX2, pY2. It assumes that  *
*         : pX1 < pX2 and pY1 < pY2. Also note that for 12 bit color       *  
*         : images the co-ordinates must be aligned to an even x           *
*         : pImageAdr is the address of the image to render                *
****************************************************************************
} 
public sub CopyImage(pX, pY as TXY, pImageAdr as word, pX1, pY1, pX2, pY2 as TXY)
   dim x, y as TXY
   dim Index, Offset as word
   dim LastPenColor as byte
   dim Image as TImage
   dim Width, Height, Range as byte
   
   // get bitmap ID, width and height...
   // get bitmap ID, width and height...
   TABLEPTR = pImageAdr
   Image.Header = ReadImageByte
   Image.Width = ReadImageByte
   Image.Height = ReadImageByte
   inc(TABLEPTR)

   // init for 8 bit color...
   LastPenColor = Pen.Color
   if Image.Is8Bit then
      Width = pX2 - pX1 + 1
      Height = pY2 - pY1 + 1
      SetRegion(pX, pY, Width, Height)
      
      Range = Width
      Offset = Image.Width - Width
      Index = 4 + pY1 * Image.Width + pX1
   
   // init for 12 bit color...
   elseif Image.Is12Bit then     
      SPICommand(cmdDatCtl)
      WriteData($00,$00,$02)

      pX = pX and $FE
      Width = (pX2 - pX1) and $FE
      Height = pY2 - pY1 + 1
      SetRegion(pX, pY, Width, Height)
      pX1 = pX1 and $FE
      inc(Width, Width / 2)
      inc(Image.Width, Image.Width / 2)
      inc(pX1, pX1 / 2)
      
      Range = Width
      Offset = Image.Width - Width
      Index = 4 + pY1 * Image.Width + pX1
   endif

   // bit blit the image...
   repeat
      Width = Range
      repeat
         TABLEPTR = pImageAdr + Index
         Pen.Color = ReadImageByte
         SPIData(Pen.Color)
         inc(index)
         dec(Width)  
      until Width = 0
      inc(Index, Offset)
      dec(Height)
   until Height = 0   

   // set back to 8 bit color...
   if Image.Is12Bit then     
      SPICommand(cmdDatCtl)
      WriteData($00,$00,$01)
   endif 
   Pen.Color = LastPenColor 
end sub 
{
****************************************************************************
* Name    : CopyImage (OVERLOAD)                                           *
* Purpose : Copies pImage to location pX, pY. The part of the image you    *
*         : to render is specified by pX1, pY1, pX2, pY2. It assumes that  *
*         : pX1 < pX2 and pY1 < pY2. Also note that for 12 bit color       *  
*         : images the co-ordinates must be aligned to an even x           *
*         : pImage is a constant array holding the image Data to render    *
****************************************************************************
} 
public sub CopyImage(pX, pY as TXY, byrefconst pImage() as byte, pX1, pY1, pX2, pY2 as TXY)
   CopyImage(pX,pY,@pImage,pX1,pY1,pX2,pY2)
end sub
{
****************************************************************************
* Name    : WriteChar                                                      *
* Purpose : Overrides the GLCD library native WriteChar call with this     *
*         : routine which is better suited for rendering to the S1D15G00   *
*         : This routine will only handle xFonts (that is, font files with *
*         : the scan line in the x direction)                              *
****************************************************************************
} 
#define GLCD_WRITECHAR // override GLCD WriteChar
#define GLCD_XFONT     // force the use of x scanned font
public sub WriteChar()
   dim FontByte as byte
   dim x, y, xRange, yRange as byte
   dim PenColor as TColor
   dim Range, Index as word
   dim Ignore as boolean
   
   PenColor = Pen.Color
   Index = 0  
   
   // is the brush style clear... 
   if Brush.Style = bsClear then
      Ignore = false
      yRange = Pos.y + Font.Height
      y = Pos.y
      while y < yRange

         MoveTo(Pos.x,y)
         xRange = Pos.x + Font.Width
         x = Pos.x
         while x < xRange
            FontByte = ReadFontByte
            
            Index = 8
            repeat
               // write a font pixel with pen color...  
               if FontByte.7 = 1 then
                  if Ignore then
                     SPICommand(cmdCaSet)
                     SPIData(x)
                     SPIData(Pos.x + Font.Width)
                     SPICommand(cmdRamWr)
                     Ignore = false
                  endif
                  SPIData(Pen.Color)
            
               // skip transparent pixel...
               else
                  Ignore = true
               endif
               FontByte = FontByte << 1
               dec(Index)
               inc(x)
            until Index = 0
         wend
         inc(y)
      wend
   
   // brush style is not clear, fast bit blit...
   else
      x = 0
      SetRegion(Pos.x, Pos.y, Font.Width, Font.Height)
      Range = Font.Height * Font.Width
      repeat
         // grab a font byte...
         if x >= Font.Width then
            FontByte = ReadFontByte
            x = 0
         elseif x mod 8 = 0 then
            FontByte = ReadFontByte
         endif 
         inc(x)  
         
         // set either font color or background color...
         if FontByte.7 = 1 then
            Pen.Color = PenColor
         else
            Pen.Color = Brush.Color
         endif
         SPIData(Pen.Color)
         FontByte = FontByte << 1
         inc(Index)
      until Index >= Range

   endif
   Pen.Color = PenColor
   inc(Pos.x,Font.Width)  
end sub

// configure the module
Initialize()

end module

